#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "importevents.h"
#include "spinboxdelegate.h"
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QSqlError>
#include <QSqlRecord>
#include <QFile>
#include <QDir>
#include <QDebug>
#include <QSqlQueryModel>
#include <QMenuBar>
#include <QMessageBox>
#include <QUrl>
#include <QWebFrame>

QSqlDatabase db;
QString sSelectedGroup = "";
QString sSelectedRound = "All";
QString sSelectedSport  = "";
QString sSelectedRegion = "";
QString sSelectedEvent = "";
QString sSelectedSeason = "";
int iSelectedSoccerDBID;
importevents *dlgImportEvents;
QHash<QString, int> hGroups;
QHash<QString, int> hTeams;
//int iEventID;

struct Standing {
    int iTeamRowID;
    QString sTeamName;
    int iPoints;
    int iWins;
    int iDraws;
    int iLosses;
    int iScored;
    int iConceeded;
} stStanding;

//QList<Standing> listStanding;
QHash<int, Standing> hStanding;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    dlgImportEvents = new importevents;
    loadMenu();
    QFile file("");
    if (existDatabase())
    {
        populateSportCombo();
//        populateEventCombo();
//        populateGroupCombo();
//        populateRoundCombo();
//        populateTeamCombo();
        sSelectedRound = "All";
        sSelectedGroup = "All";
        displayGames();
        showStandings();
    }

    ui->webView_2->settings()->setAttribute(QWebSettings::JavascriptEnabled, true);
    ui->webView->settings()->setAttribute(QWebSettings::JavascriptEnabled, true);
    showOnlineLeague(714); //Italy - Serie A - 2009-10
}

void MainWindow::showOnlineLeague(int iSelectedSoccerDBID)
{
    QString sHTML;
    QString sSelectedSoccerDBID;
    sSelectedSoccerDBID.setNum(iSelectedSoccerDBID);

    sHTML = "<HTML><script type='text/javascript' src='http://www.aragon.ws/soccerdb/getmatches.php?id=" + sSelectedSoccerDBID + "&date=latest'></script></HTML>";
    qDebug() << "sHTML :" << sHTML ;
    ui->webView->setHtml(sHTML, QUrl::fromLocalFile(QDir::currentPath()));
    ui->webView->show();
    qDebug() << "webView:" << ui->webView->page()->mainFrame()->toHtml();

    ui->webView_2->settings()->setAttribute(QWebSettings::JavascriptEnabled, true);
    sHTML = "<HTML><script type='text/javascript' src='http://www.aragon.ws/soccerdb/gettable.php?id=" + sSelectedSoccerDBID + "'></script></HTML>";
    ui->webView_2->setHtml(sHTML, QUrl::fromLocalFile(QDir::currentPath()));
    ui->webView_2->show();
    qDebug() << "webView_2:" << ui->webView_2->page()->mainFrame()->toHtml();

}

bool MainWindow::existDatabase()
{

    QFile fDbFile;
    QString path;
    bool existDb;

#ifdef Q_OS_LINUX
        // NOTE: We have to store database file into user home folder in Linux
//            QString path(QDir::home().path());
        path = QDir::home().path();
        path.append(QDir::separator()).append("qsportsevent.db");
        path = QDir::toNativeSeparators(path);
#else
        path = "./qsportsevent.db" ;
        // NOTE: File exists in the application private folder, in Symbian Qt implementation
#endif
        fDbFile.setFileName(path);
        qDebug() << "Db path:" << path;
        qDebug() << "fDbFile.exists():" << fDbFile.exists();
        existDb = fDbFile.exists();

        if (existDb)
        {
            QSqlDatabase db;
            db = QSqlDatabase::addDatabase("QSQLITE");
            db.setDatabaseName(path);
            if (!db.open())
            {
                qDebug() << "Failed to connect";
                qDebug() << db.lastError();
            }
        }

        return existDb ;
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::loadMenu()
{
    QMenuBar *menuBar=new QMenuBar();
    QAction *showImportEvents=new QAction("Import Event",this);
    menuBar->addAction(showImportEvents);
    connect(showImportEvents,SIGNAL(triggered()),this,SLOT(showImportEvents()));
    QAction *showAbout=new QAction("About",this);
    menuBar->addAction(showAbout);
    connect(showAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
    setMenuBar(menuBar);
}

void MainWindow::showImportEvents()
{

    dlgImportEvents->show();
    if (dlgImportEvents->exec() == QDialog::Accepted)
    {
    }
//    populateEventCombo();
    populateSportCombo();
//    populateGroupCombo();
//    populateRoundCombo();
//    populateTeamCombo();
//    displayGames();
//    showStandings();
}


void MainWindow::displayGames()
{
    QSqlQuery qry;
    QString sql;
    QString sqlAppend1;
    QString sqlAppend2;
//    QSqlQueryModel *model = new QSqlQueryModel;
    QString str;
 //   QString sql ="SELECT * FROM Games ORDER BY gametime ASC";
/*
    QString sql ="SELECT games.gametime, games.venue, hometeam.name, awayteam.name, games.homegoals, games.awaygoals "
                 "FROM Games, teamgroups, teams as hometeam, teams as awayteam "
                 "WHERE games.awayteamrowid = awayteam.rowid "
                 "AND  games.hometeamrowid = hometeam.rowid "
                 "AND games.grouprowid = teamgroups.rowid "
                 "ORDER BY teamgroups.name, games.gametime ASC";
*/
//                 "WHERE games.awayteamrowid = awayteam.rowid ";
//                 "AND  games.hometeamrowid = hometeam.rowid "
//                 "AND games.grouprowid = teamgroups.rowid "
//                "ORDER BY teamgroups.name, games.gametime ASC";
//    QTableView *view;
//    tbwGames
/*
    sql ="SELECT * "
                 "FROM Games ";
    model->setQuery(sql,db); // DB is a QSqlDatabase object
    qDebug() << "lastError" << model->lastError();
    ui->tbwGames->setModel(model);

    QSqlQueryModel *model2 = new QSqlQueryModel;
    sql ="SELECT rowid, * from Games";
    model2->setQuery(sql,db); // DB is a QSqlDatabase object
    qDebug() << "lastError" << model2->lastError();
    ui->tbvGames->setModel(model2);

    QSqlQueryModel *model3 = new QSqlQueryModel;
    sql ="SELECT rowid, * from Teams";
    model3->setQuery(sql,db); // DB is a QSqlDatabase object
    qDebug() << "lastError" << model3->lastError();
    ui->tbvTeams->setModel(model3);

    QSqlQueryModel *model4 = new QSqlQueryModel;
    sql ="SELECT rowid, * from teamgroups";
    model4->setQuery(sql,db); // DB is a QSqlDatabase object
    qDebug() << "lastError" << model4->lastError();
    ui->tbvGroups->setModel(model4);
*/



    sql = "SELECT rowid, name from teamgroups "
          "WHERE eventrowid=:eventrowid";

    qry.prepare(sql);
    qry.bindValue("eventrowid:", iEventID);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.lastError();
    }

    while (qry.next()) {
        //Default group is the first one.
        if (sSelectedGroup == "")
        {
            sSelectedGroup = qry.value(1).toString();
        }
        hGroups[qry.value(1).toString()] = qry.value(0).toInt();
        qDebug() << qry.value(0).toString();
        qDebug() << qry.value(1).toString();
    }

    qDebug() << sSelectedGroup;

    sql = "SELECT rowid, name from teams ";
    if (sSelectedGroup != "All")
    {
        sqlAppend1 = " WHERE grouprowid=" + QString::number(hGroups.value(sSelectedGroup),10);
        qDebug() << sqlAppend1;
        qDebug() << hGroups.value(sSelectedGroup);
    }

    qry.prepare(sql + sqlAppend1);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.lastError();
    }

    hTeams.clear();
    hStanding.clear();
    qDebug() << hTeams.keys();
    qDebug() << hTeams.values();

    while (qry.next()) {
        Standing stTeam;
        hTeams[qry.value(1).toString()] = qry.value(0).toInt();
        stTeam.sTeamName = qry.value(1).toString();
        stTeam.iPoints = 0;
        stTeam.iWins = 0;
        stTeam.iDraws = 0;
        stTeam.iLosses = 0;
        stTeam.iScored = 0;
        stTeam.iConceeded = 0;
        stTeam.iTeamRowID = qry.value(0).toInt();
        hStanding[qry.value(0).toInt()] = stTeam;
        qDebug() << qry.value(1).toString() << qry.value(0).toString();
        qDebug() << hTeams.keys();
        qDebug() << hTeams.values();
    }

    sqlAppend1="";
    sqlAppend2="";
    sql ="SELECT grouprowid, gametime, hometeamrowid, awayteamrowid, homegoals, awaygoals, round from games ";
    if (sSelectedGroup != "All")
    {
        sqlAppend1 = " grouprowid=" + QString::number(hGroups.value(sSelectedGroup),10);
        qDebug() << sqlAppend1;
        qDebug() << hGroups.value(sSelectedGroup);
    }

    if (sSelectedRound != "All")
    {
        sqlAppend2 = " round='" + sSelectedRound + "'";
    }

    if (sqlAppend1 != "" && sqlAppend2 != "")
    {
        sql = sql + " WHERE " + sqlAppend1 + " AND " + sqlAppend2;
    }
    else if (sqlAppend1 != "")
    {
        sql = sql + " WHERE " + sqlAppend1 ;
    }
    else if (sqlAppend2 != "")
    {
        sql = sql + " WHERE " + sqlAppend2;
    }

    qry.prepare(sql);

    qDebug() << sql;

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.lastError();
    }

    qDebug() << "qry.size()" << qry.size();
    ui->twGames->clear();
//    ui->twGames->setAlternatingRowColors(true);

    ui->twGames->setRowCount(48);//qry.size());
    ui->twGames->setColumnCount(6);

    qDebug() << hTeams.keys();
    qDebug() << hTeams.values();
    qDebug() << hGroups.keys();
    qDebug() << hGroups.values();
    int row=0;
    while (qry.next()) {
        for (int column = 0; column < 6; column++)
        {
            switch (column)
            {
            case 0:
                qDebug() << "Group:" << qry.value(column).toString();
                str = hGroups.key(qry.value(column).toInt());
                qDebug() << "str" << str;
                break;
            case 2:
            case 3:
                qDebug() << "Team:" << qry.value(column).toString();
                str = hTeams.key(qry.value(column).toInt());
                qDebug() << "str" << str;
                break;
            case 1:
            case 4:
            case 5:
                qDebug() << "item" << qry.value(column).toString();
                str = qry.value(column).toString();
                break;
            }
            QTableWidgetItem *newItem = new QTableWidgetItem(str);
            ui->twGames->setItem(row, column, newItem);
        }
        row++;
     }
    ui->twGames->resizeColumnsToContents();
    ui->twGames->setItemDelegateForColumn(4, new SpinBoxDelegate(ui->twGames));
    ui->twGames->setItemDelegateForColumn(5, new SpinBoxDelegate(ui->twGames));
    ui->twGames->setColumnWidth(4,40);
    ui->twGames->setColumnWidth(5,40);
}

void MainWindow::populateGroupCombo()
{
    QSqlQuery qry;

    qry.prepare("SELECT rowid, name "
                 "FROM teamgroups "
                 "WHERE eventrowid=:eventrowid");

    qry.bindValue(":eventrowid", iEventID);
    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();

    ui->cbGroups->clear();
    ui->cbGroups->addItem("All");
    while (qry.isValid())
    {
        ui->cbGroups->addItem(qry.value(1).toString());
        qry.next();
    }
}

void MainWindow::populateSportCombo()
{
    QSqlQuery qry;

    qry.prepare("SELECT DISTINCT sport "
                 "FROM events ");

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();

    ui->cbSport->clear();
    while (qry.isValid())
    {
        ui->cbSport->addItem(qry.value(0).toString());
        qry.next();
    }
}

void MainWindow::populateRegionCombo(QString sSport)
{
    QSqlQuery qry;

    qry.prepare("SELECT DISTINCT region "
                 "FROM events "
                 "WHERE sport=:sport");
    qry.bindValue(":sport", sSport);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();

    ui->cbRegion->clear();
    while (qry.isValid())
    {
        ui->cbRegion->addItem(qry.value(0).toString());
        qry.next();
    }
}

void MainWindow::populateTeamCombo()
{
    QSqlQuery qry;
    QString sql;
    QString sqlAppend;

    sql = "SELECT rowid, name "
          "FROM teams ";

    if (sSelectedGroup != "All")
    {
        sqlAppend = "WHERE grouprowid=" + QString::number(hGroups.value(sSelectedGroup),10);
        qDebug() << sqlAppend;
        qDebug() << hGroups.value(sSelectedGroup);
    }

    qry.prepare(sql + sqlAppend);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();

    ui->cbTeam->clear();
    ui->cbTeam->addItem("All");
    while (qry.isValid())
    {
        ui->cbTeam->addItem(qry.value(1).toString());
        qry.next();
    }
}

void MainWindow::populateRoundCombo()
{
    QSqlQuery qry;
    int iMaxRound;

    qry.prepare("SELECT MAX(round) as maxround "
                 "FROM games ");

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();
    iMaxRound = qry.value(0).toInt();
    ui->cbRounds->clear();
    ui->cbRounds->addItem("All");

    for (int i=1; i<iMaxRound+1; i++)
    {
        ui->cbRounds->addItem(QString::number(i,10));
    }

}

void MainWindow::populateSeasonCombo(QString sSport, QString sRegion, QString sEventName)
{
    QSqlQuery qry;

    qry.prepare("SELECT DISTINCT season "
                 "FROM events "
                 "WHERE sport=:sport and region=:region and name=:name");
    qry.bindValue(":sport", sSport);
    qry.bindValue(":region", sRegion);
    qry.bindValue(":name", sEventName);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();
    ui->cbSeason->clear();

    while (qry.isValid())
    {
        ui->cbSeason->addItem(qry.value(0).toString());
        qry.next();
    }

}

void MainWindow::populateEventCombo(QString sSport, QString sRegion)
{
    QSqlQuery qry;
//    bool bFirst = true;
    qry.prepare("SELECT DISTINCT name "
                 "FROM events "
                 "WHERE sport=:sport and region=:region");
    qry.bindValue(":sport", sSport);
    qry.bindValue(":region", sRegion);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.size();
        qDebug() << qry.lastError();

    }
    qry.next();

    ui->cbEvents->clear();
    while (qry.isValid())
    {
/*        if (bFirst)
        {
            iEventID = qry.value(0).toInt() ;
            bFirst = false;
        }
*/
        ui->cbEvents->addItem(qry.value(0).toString());
        qry.next();
    }

}

void MainWindow::on_cbSport_activated(QString sSportName)
{
    sSelectedSport = sSportName;
    populateRegionCombo(sSportName);
//    displayGames();
//    showStandings();
}

void MainWindow::on_cbRegion_activated(QString sRegionName)
{
    sSelectedRegion = sRegionName;
    populateEventCombo(sSelectedSport, sSelectedRegion);
//    displayGames();
//    showStandings();
}

void MainWindow::on_cbEvents_activated(QString sEventName)
{
    sSelectedEvent = sEventName;
    populateSeasonCombo(sSelectedSport, sSelectedRegion, sSelectedEvent);
//    displayGames();
//    showStandings();
}

void MainWindow::on_cbGroups_activated(QString sGroupName)
{
    sSelectedGroup = sGroupName;
    displayGames();
    showStandings();
}

void MainWindow::on_cbSeason_activated(QString sSeason)
{
    QSqlQuery qry;
    QString sql;

    sSelectedSeason = sSeason;
    sql = "SELECT rowid, soccerdbid from events "
          "WHERE name = :name and season = :season";
    qry.prepare(sql);
    qry.bindValue(":name", sSelectedEvent);
    qry.bindValue(":season", sSelectedSeason);

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.lastError();
    }

    if (qry.next()) {
        iEventID = qry.value(0).toInt();
        iSelectedSoccerDBID = qry.value(1).toInt();
    }
    else
    {
        iEventID = -1;
    }

    showOnlineLeague(iSelectedSoccerDBID);
    populateGroupCombo();
    populateRoundCombo();
    displayGames();

}

void MainWindow::on_cbRounds_activated(QString sRound)
{
    sSelectedRound = sRound;
    displayGames();
}

void MainWindow::showAbout()
{
    QString message = "";
    QString version;

//    message.append(APP_NAME);
    message.append("QSportsEvent");
    message.append("-");
//    version = version.setNum(APP_VERSION);
    version = "0.0.1";
    message.append(version);
    message.append("\n by ");
//    message.append(AUTHOR);
    message.append("Magnus Lundgren (magnuslu@hotmail.com)");
    QMessageBox::about ( this, tr("About"), message);
}

void MainWindow::showStandings()
{
    QSqlQuery qry;
    QString sql;
    int iHomeTeamID;
    int iAwayTeamID;
    int iHomeGoals;
    int iAwayGoals;
    Standing stHomeTeam;
    Standing stAwayTeam;
    QTableWidgetItem *newItem;

    if (sSelectedGroup == "All" || sSelectedGroup == "")
    {
        qDebug() << "Can't show standings for All groups" << sSelectedGroup;
        return;
    }


    sql ="SELECT hometeamrowid, awayteamrowid, homegoals, awaygoals from games "
         "WHERE grouprowid=" + QString::number(hGroups.value(sSelectedGroup),10);

//    str = hGroups.key(qry.value(column).toInt());
    qry.prepare(sql);
    qDebug() << sql;

    if (!qry.exec())
    {
        qDebug() << qry.lastQuery();
        qDebug() << qry.lastError();
    }
    while (qry.next()) {
        iHomeTeamID = qry.value(0).toInt();
        iAwayTeamID = qry.value(1).toInt();
        iHomeGoals = qry.value(2).toInt();
        iAwayGoals = qry.value(3).toInt();
        stHomeTeam = hStanding.value(iHomeTeamID);
        stAwayTeam = hStanding.value(iAwayTeamID);
        qDebug() << iHomeTeamID << iAwayTeamID << iHomeGoals << iAwayGoals;
        if (iHomeGoals > iAwayGoals)
        {
            stHomeTeam.iPoints=stHomeTeam.iPoints + 3;
            stHomeTeam.iWins++;
            stAwayTeam.iLosses++;
        }
        else if (iHomeGoals < iAwayGoals)
        {
            stAwayTeam.iPoints=stAwayTeam.iPoints + 3;
            stHomeTeam.iLosses++;
            stAwayTeam.iWins++;
        }
        else
        {
            stHomeTeam.iPoints=stHomeTeam.iPoints + 1;
            stAwayTeam.iPoints=stAwayTeam.iPoints + 1;
            stHomeTeam.iDraws++;
            stAwayTeam.iDraws++;
        }

        stHomeTeam.iScored=stHomeTeam.iScored + iHomeGoals;
        stHomeTeam.iConceeded=stHomeTeam.iConceeded + iAwayGoals;
        stAwayTeam.iScored=stAwayTeam.iScored + iAwayGoals;
        stAwayTeam.iConceeded=stAwayTeam.iConceeded + iHomeGoals;

        qDebug() << stHomeTeam.sTeamName << stHomeTeam.iPoints;
        qDebug() << stAwayTeam.sTeamName << stAwayTeam.iPoints;
        hStanding[iHomeTeamID] = stHomeTeam;
        hStanding[iAwayTeamID] = stAwayTeam;
        }

    int j=0;
    QHash<int, Standing>::iterator i;

    ui->twStandings->clear();
    ui->twStandings->setRowCount(hStanding.count());
    ui->twStandings->setColumnCount(9);


    for (i = hStanding.begin(); i != hStanding.end(); ++i)
    {
        stAwayTeam = i.value();

        //Team Name
        newItem = new QTableWidgetItem(stAwayTeam.sTeamName);
        qDebug() << j << stAwayTeam.sTeamName;
        ui->twStandings->setItem(j, 0, newItem);

        //Games played
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iWins +
                                                       stAwayTeam.iDraws +
                                                       stAwayTeam.iLosses,10));
        qDebug() << j << QString::number(stAwayTeam.iPoints,10);
        ui->twStandings->setItem(j, 1, newItem);

        //Games won
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iWins,10));
        qDebug() << j << QString::number(stAwayTeam.iWins,10);
        ui->twStandings->setItem(j, 2, newItem);

        //Games drawn
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iDraws,10));
        qDebug() << j << QString::number(stAwayTeam.iDraws,10);
        ui->twStandings->setItem(j, 3, newItem);

        //Games lost
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iLosses,10));
        qDebug() << j << QString::number(stAwayTeam.iLosses,10);
        ui->twStandings->setItem(j, 4, newItem);

        //Goals for
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iScored,10));
        qDebug() << j << QString::number(stAwayTeam.iScored,10);
        ui->twStandings->setItem(j, 5, newItem);

        //Goals against
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iConceeded,10));
        qDebug() << j << QString::number(stAwayTeam.iConceeded,10);
        ui->twStandings->setItem(j, 6, newItem);

        //Goal difference
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iScored-stAwayTeam.iConceeded,10));
        qDebug() << j << QString::number(stAwayTeam.iScored-stAwayTeam.iConceeded,10);
        ui->twStandings->setItem(j, 7, newItem);

        //Points
        newItem = new QTableWidgetItem(QString::number(stAwayTeam.iPoints,10));
        qDebug() << j << QString::number(stAwayTeam.iPoints,10);
        ui->twStandings->setItem(j, 8, newItem);

        j++;
    }
    ui->twStandings->resizeColumnsToContents();
}
